/*	commsISRs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define COMMSISRS_C
#include "inc/freeEMS.h"
#include "inc/interrupts.h"
#include "inc/utils.h"
#include "inc/commsISRs.h"


#include "inc/flashWrite.h"      /* Testing ONLY*/


/* SCI0 ISR handles all interrupts for SCI0 by reading the flag registers and acting appropriately. */
void SCI0ISR(void)
{
	/* Read the flags register */
	unsigned char flags = SCI0SR1;

	/* From left most/most significant to right most/least significant.
	 *
	 *     Abrev  Description					  Actions
	 * 1 = TRDE - Transmit Data Register Empty	- load with more data if there is any
	 * 1 = TC   - Transmit Complete				- turn flow back on
	 * 1 = RDRF - Receive Data Register Full	- Store data in buffer until packet complete.
	 * 1 = IDLE - Idle Line						- transmit datalogging if available
	 * 1 = OR   - Overrun						- discard partial data received and return error packet (code for overrun error) if configured to
	 * 1 = NF   - Noise Flag					- accept data, increment counter, send error packet if configured to
	 * 1 = FE   - Framing Error					- discard partial data received and return error packet (code for framing error) if configured to
	 * 1 = PF   - Parity Error					- discard data sent and return negative ack (code for bad parity) or error packet (same code) if configured to, increment counter
	 */

	// fix warning
	flags++;
	// nested interrupts here? (reenable isrs before finishing to allow the engine to run best.)

	//if(material to send && character buffer open){
	//	send char and wait till next char slot is free.
	//}

	// loop sending chars from a buffer of some sort as available/time permits.
}


/* Send string over serial with length */
void sendChar(char *str, unsigned char length){
	int i;
	for (i=0;i<length;i++){
		// poll trde flag and send a byte when ready
		while(!(SCI0SR1 & 0x80)){/* Wait for ever until able to send then move on */}
		SCI0DRL = str[i];
	}
}


/* Take a pointer and a byte and return a new pointer after writing the escaped byte into ram */
void escapeAndSendByte(unsigned char value, unsigned char address){
	// TODO fix/finish this
	if(value == ESCAPE_BYTE){
		// write escape byte
	//	*address = ESCAPE_BYTE;
		address++;
		// write escaped escape byte
		//*address = ESCAPED_ESCAPE_BYTE;
		address++;
		//return address;
	}else if(value == START_BYTE){
		// write escape byte
		//*address = ESCAPE_BYTE;
		address++;
		// write escaped start byte
		//*address = ESCAPED_START_BYTE;
		address++;
		//return address;
	}else if(value == STOP_BYTE){
		// write escape byte
		//*address = ESCAPE_BYTE;
		address++;
		// write escaped stop byte
		//*address = ESCAPED_STOP_BYTE;
		address++;
		//return address;
	}else{
		// write byte
		//*address = value;
		address++;
		//return address;
	}
}


void sendMainTable(){}


/* Send temp HR loggin data */
void sendHRLog(){

	unsigned short commsStartTime = TCNT;

	char winEOL[3] = {0x0D,0x0A,0}; // (cr lf pair with null termination)

	/* Comment out and add in vars as you wish.
	 *
	 * The only requirements are that you put the length of your string in correctly
	 * and use the correct send function for the data type required. Using the Long
	 * one for all types should work, but would be slow, so there are 3 types for now.
	 *
	 * Unsigned Char
	 * Unsigned Short
	 * Unsigned Long
	 *
	 * Signed numbers can not be displayed correctly, though this could be added fairly easily.
	 */


	// Serial experimentation
	send("Logs=");
	sendUS(Counters.logsSentCounter);
	send(", Sec=");
	sendUS(Clocks.realTimeClockSeconds);

	/* Test region at front for visibility */

	/* various temporary variables that need tidying and replacing with the real macoy */
//	send(", ECPT=");
//	sendUL(engineCyclePeriod);
//	send(", MPW="); // Temporary pulse width to be used for all cyls
//	sendUS(masterPulseWidth);
//	send(", DFRN="); // Delay From Reference iNjection
//	sendUS(totalAngleAfterReferenceInjection);
//	send(", DFRG = ");	// dwell and advance currently match fuel for comparative purposes...
//	sendUS(totalAngleAfterReferenceIgnition);
//	send(", LCKS="); // Lost Crank Sync
//	sendUS(Counters.lostCrankSyncCounter);
//	send(", PPPSP="); // Primary teeth between secondary teeth
//	sendUS(primaryPulsesPerSecondaryPulseBuffer);


	/* Not currently under test */
	// not likely to occur on the new test rig, but good to have in-car
//	send(", LowVolts = ");
//	sendUC(lowVoltISRCounter);
	// not currently used.
//	send(", LostCamSync = ");
//	sendUC(lostCamSyncCounter);
	// seems to be working well
//	send(", tacho=");
//	sendUS(tachoPeriod);


	/* Core variables */
	send(", BinRPM=");
	sendUS(CoreVars.RPM[recordADCBank]); // TODO why why why is this correct??
	send(", HR RPM=");
	sendUS(CoreVars.RPM[recordADCBank]/2); // TODO why why why is this correct??
	send(", IAT=");
	sendUS(CoreVars.IAT[recordADCBank]);
	send(", CHT=");
	sendUS(CoreVars.CHT[recordADCBank]);
	send(", TPS=");
	sendUS(CoreVars.TPS[recordADCBank]);
	send(", EGO=");
	sendUS(CoreVars.EGO[recordADCBank]);
	send(", BRV=");
	sendUS(CoreVars.BRV[recordADCBank]);
	send(", MAP=");
	sendUS(CoreVars.MAP[recordADCBank]);
	send(", AAP=");
	sendUS(CoreVars.AAP[recordADCBank]);
	send(", MAT=");
	sendUS(CoreVars.MAT[recordADCBank]);

	// no pots attached to second bank of ADCS
//	send(", EGO2=");
//	sendUS(CoreVars.EGO2);
//	send(", IAP=");
//	sendUS(CoreVars.IAP);
//	send(", =");
//	sendUS(CoreVars.MAF);

	// no fixed use for these inputs yet
//			send(", SP3=");
//			sendUS(localSP3);
//			send(", SP4=");
//			sendUS(localSP4);
//			send(", SP5=");
//			sendUS(localSP5);
//			send(", SP6=");
//			sendUS(localSP6);
//			send(", SP7=");
//			sendUS(localSP7);

	/* Calculated stuff */
	send(", Load=");
	sendUS(DerivedVars.LoadMain[realTimeUseBank]);
	send(", VE=");
	sendUS(DerivedVars.VEMain[realTimeUseBank]);
	send(", Lambda=");
	sendUS(DerivedVars.Lambda[realTimeUseBank]);
//	send(", topAF=");
//	sendUS(DerivedVars.AirFlow[realTimeUseBank]);
//	send(", bottom=");
//	sendUS(DerivedVars.densityAndFuel[realTimeUseBank]);
	send(", BPW=");
	sendUS(DerivedVars.BasePW[realTimeUseBank]);
//	send(", IDT=");
//	sendUS(DerivedVars.IDT[realTimeUseBank]);
//	send(", ETE=");
//	sendUS(DerivedVars.ETE[realTimeUseBank]);
//	send(", TFC=");
//	sendSS(DerivedVars.TFCTotal[realTimeUseBank]);
//	send(", FPW=");
//	sendUS(DerivedVars.FinalPW[realTimeUseBank]);
//	send(", RPW=");
//	sendUS(DerivedVars.RefPW[realTimeUseBank]);

	/* Calcs run time */
	/* Math/Input routines runtimes */
//	send(", CoRT=");
//	sendUS(RuntimeVars.genCoreVarsRuntime);
//	send(", DeRT=");
//	sendUS(RuntimeVars.genDerivedVarsRuntime);
//	send(", CaRT=");
//	sendUS(RuntimeVars.calcsRuntime);
//	send(", MtRT=");
//	sendUS(RuntimeVars.mathTotalRuntime);
//	send(", MsRT=");
//	sendUS(RuntimeVars.mathSumRuntime);

	/* Samples to calcs comparison */
//	send(", ADCC=");
//	sendUS(Counters.syncedADCreadingCounter);
//	send(", TIOC=");
//	sendUS(Counters.timeoutADCreadingCounter);
//	send(", TOTC=");
//	sendUS(Counters.timeoutADCreadingCounter + Counters.syncedADCreadingCounter);
//	send(", Calcs=");
//	sendUS(Counters.calcsPerformedCounter);

//	send(", FlashCLock=");
//	sendUS(FCLKDIV);
//	send(", FlashSTAT=");
//	sendUC(FSTAT);
//	send(", FlashPROT=");
//	sendUC(FPROT);
//    send(", FlashCMD=");
//	sendUC(FCMD);
//
//		if (Counters.calcsPerformedCounter == 3){
//			send(", READY??");
//			send(", FlashSTAT=");
//				sendUC(FSTAT);
//			send(", FlashCMD=");
//				sendUC(FCMD);
//
//			send(", GO!! Calling eraseSector  ");
//			eraseSector((unsigned short*) 0x2800 );    // is loaded into register D
//
//			send(", GO!! Calling writeWord   ");
//			writeWord((unsigned short*) 0x2800, (unsigned short) 0x0006 );
//			// the first argument is loaded into register D,
//			//the second argument into register W
//
//			//		eraseSector((tribyte*)0x4095); //4095 is loaded into register D
//		//	send(", FlashSTAT=");
//		//		sendUC(FSTAT);
//		//	send(", FlashCMD=");
//		//	 	sendUC(FCMD);
//
//		}

//	send(", LTime=");
//	sendUS(RuntimeVars.logSendingRuntime);
//	send(", pilr=");
//	sendUS(RuntimeVars.primaryInputLeadingRuntime);
//	send(", pitr=");
//	sendUS(RuntimeVars.primaryInputTrailingRuntime);
//	send(", silt=");
//	sendUS(RuntimeVars.secondaryInputLeadingRuntime);
//	send(", sitt=");
//	sendUS(RuntimeVars.secondaryInputTrailingRuntime);
//	send(", drop=");
//	sendUS(Counters.primaryTeethDroppedFromLackOfSync);
	// latencies, runtimes, etc


	/* crlf pair to terminate log */
	send(winEOL);
	Counters.logsSentCounter++;
	RuntimeVars.logSendingRuntime = TCNT - commsStartTime;
}

// TODO Some of this should possibly be moved to the ISR, this is just a demo/experimental work for
// TODO getting data out in HR form for now. Soon it will be gutted and the serial spec implemented.


/* Send block of data over serial by address and length */
void sendBlock(unsigned short address, unsigned short length){
	unsigned char *block = (unsigned char*)address;
	int i;
	for (i=0;i<length;i++){
		// poll trde flag and send a byte when ready
		while(!(SCI0SR1 & 0x80)){/* Wait for ever until able to send then move on */}
		SCI0DRL = block[i];
	}
}


/* Send string over serial with length */
void sendSt(char *str, unsigned char length){
	int i;
	for (i=0;i<length;i++){
		// poll trde flag and send a byte when ready
		while(!(SCI0SR1 & 0x80)){/* Wait for ever until able to send then move on */}
		SCI0DRL = str[i];
	}
}


/* Send string over serial */
void send(char *str){
	int i = 0;
	do{
		// poll trde flag and send a byte when ready
		while(!(SCI0SR1 & 0x80)){/* Wait for ever until able to send then move on */}
		SCI0DRL = str[i];
		i++;
	}while(str[i] != '\0');
}


/* Send generic error message with code */
void sendError(unsigned short code){
	send("Error : ");
	sendUS(code);
}


/* Send unsigned short over serial */
void sendUC(unsigned char value)
{
	char str[3];		// ascii number holder
	itoaUC(value, str);	// convert from unsigned char to ascii
	sendSt(str, 3);		// send the string on the serial line
}


/* Send unsigned short over serial */
void sendUS(unsigned short value)
{
	char str[5];		// ascii number holder
	itoaUS(value, str);	// convert from unsigned short to ascii
	sendSt(str, 5);		// send the string on the serial line
}


/* Send unsigned short over serial */
void sendSS(signed short value)
{
	unsigned short usn;
    if(value < 0){
    	send("-");
    	usn = (unsigned short)(-1 * value);
    }else{
    	usn = (unsigned short)value;
    }

	char str[5];		// ascii number holder
	itoaUS(usn, str);	// convert from unsigned short to ascii
	sendSt(str, 5);		// send the string on the serial line
}


/* Send unsigned short over serial */
void sendUL(unsigned long value)
{
	char str[10];		// ascii number holder
	itoaUL(value, str);	// convert from unsigned long to ascii
	sendSt(str, 10);	// send the string on the serial line
}


//	To initiate an SCI transmission:
//		1. Configure the SCI: (in init.c only)
//			a)	Select a baud rate. Write this value to the SCI baud registers (SCIBDH/L) to begin the baud
//				rate generator. Remember that the baud rate generator is disabled when the baud rate is zero.
//				Writing to the SCIBDH has no effect without also writing to SCIBDL.
//			b)	Write to SCICR1 to configure word length, parity, and other configuration bits
//				(LOOPS,RSRC,M,WAKE,ILT,PE,PT).
//			c)	Enable the transmitter, interrupts, receive, and wake up as required, by writing to the SCICR2
//				register bits (TIE,TCIE,RIE,ILIE,TE,RE,RWU,SBK). A preamble or idle character will now
//				be shifted out of the transmitter shift register.
//		2. Transmit Procedure for Each Byte: (here)
//			a)	Poll the TDRE flag by reading the SCISR1 or responding to the TDRE interrupt. Keep in mind
//				that the TDRE bit resets to one.
//			b)	If the TDRE flag is set, write the data to be transmitted to SCIDRH/L, where the ninth bit is
//				written to the T8 bit in SCIDRH if the SCI is in 9-bit data format. A new transmission will not
//				result until the TDRE flag has been cleared.
//		3. Repeat step 2 for each subsequent transmission.
