/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define MAIN_OR_GLOBALS
#include "inc/main.h"

int  main(){
	// Set everything up.
	init();

	//LongNoTime.timeLong = 54;
	// Run forever repeating.
	while(TRUE){
	//	unsigned short start = realTimeClockMillis;
		/* If ADCs require forced sampling, sample now */
		if(coreStatusA & FORCE_READING){
			ATOMIC_START(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
			/* Atomic block to ensure a full set of readings are taken together */

			/* Check to ensure that a reading wasn't take before we entered a non interruptable state */
			if(coreStatusA & FORCE_READING){ // do we still need to do this TODO ?

//				sampleEachADC(&ADCArrays); TODO still need to do a pair of loops and clock these two functions for performance.
				sampleLoopADC(&ADCArrays);
				resetToNonRunningState();
				Counters.timeoutADCreadingCounter++;

				/* Set flag to say calc required */
				coreStatusA |= CALC_FUEL_IGN;

				/* Clear force reading flag */
				coreStatusA &= ~FORCE_READING;
			}

			ATOMIC_END(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
		}

		/* If required, do main fuel and ignition calcs first */
		if(coreStatusA & CALC_FUEL_IGN){
			ATOMIC_START(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
			/* Atomic block to ensure that we don't clear the flag for the next data set when things are tight */

			/* Switch input bank so that we have a stable set of the latest data */
			if(recordADCBank){
				recordADCBank = 0;
				mathInputBank = 1;
			}else{
				recordADCBank = 1;
				mathInputBank = 0;
			}

			/* Clear the calc required flag */
			coreStatusA &= ~CALC_FUEL_IGN;

			ATOMIC_END(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

			/* Store the latency from sample time to runtime */
			ISRLatencyVars.mathLatency = TCNT - ISRLatencyVars.mathSampleTimeStamp[mathInputBank];
			/* Keep track of how many calcs we are managing per second... */
			Counters.calcsPerformedCounter++;
			/* ...and how long they take each */
			unsigned short mathStartTime = TCNT;

			/* Generate the core variables from sensor input and recorded tooth timings */
			generateCoreVars();

			RuntimeVars.genCoreVarsRuntime = TCNT - mathStartTime;
			unsigned short derivedStartTime = TCNT;

			/* Generate the derived variables from the core variables based on settings */
			generateDerivedVars();

			RuntimeVars.genDerivedVarsRuntime = TCNT - derivedStartTime;
			unsigned short calcsStartTime = TCNT;

			/* Perform the calculations TODO possibly move this to the software interrupt if it makes sense to do so */
			calculateFuelAndIgnition();

			RuntimeVars.calcsRuntime = TCNT - calcsStartTime;
			/* Record the runtime of all the math total */
			RuntimeVars.mathTotalRuntime = TCNT - mathStartTime;

			RuntimeVars.mathSumRuntime = RuntimeVars.calcsRuntime + RuntimeVars.genCoreVarsRuntime + RuntimeVars.genDerivedVarsRuntime;

			ATOMIC_START(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
			/* Atomic block to ensure that outputBank and outputBank Offsets match */

			/* Switch banks to the latest data */
			if(mathInternalBank){
				mathInternalBank = 0;
				mathInternalInjectionOffset = 0;
				mathInternalIgnitionOffset = 0;
				realTimeUseBank = 1;
				realTimeUseInjectionOffset = INJECTION_CHANNELS;
				realTimeUseIgnitionOffset = IGNITION_CHANNELS;
			}else{
				mathInternalBank = 1;
				mathInternalInjectionOffset = INJECTION_CHANNELS;
				mathInternalIgnitionOffset = IGNITION_CHANNELS;
				realTimeUseBank = 0;
				realTimeUseInjectionOffset = 0;
				realTimeUseIgnitionOffset = 0;
			}

			ATOMIC_END(); /*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
		}else{
			/* In the event that no calcs are required, sleep a little before returning to retry. */
			sleepMicro(RuntimeVars.mathTotalRuntime); // not doing this will cause the ISR lockouts to run for too high a proportion of the time
			/* Using 0.8 ticks as micros so it will run for a little longer than the math did */
		}

		unsigned short logTimeBuffer = Clocks.realTimeClockTenths;

		// send logging once per chosen clock tick
		if((lastTime != logTimeBuffer) && (lastCalcCount != Counters.calcsPerformedCounter)){
			sendHRLog();

			// mechanism to ensure we send once per clock tick without doing it in the RTC section.
			lastTime = logTimeBuffer;
			// mechanism to ensure we only send something if the data has been updated
			lastCalcCount = Counters.calcsPerformedCounter;
		}else{
			/* Balance out the loop for consistent running */
			sleepMicro(RuntimeVars.logSendingRuntime);
		}

		// on once per cycle for main loop heart beat (J0)
		PORTJ ^= 0x01;

		// PWM experimentation
		adjustPWM();
	}
}
