/*	utils.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define UTILS_C
#include "inc/freeEMS.h"
#include "inc/commsISRs.h"
#include "inc/utils.h"


/* Do table switching based on boolean parameter */
void setupPagedRAM(unsigned char bool){
	if(bool){
		currentFuelRPage = RPAGE_FUEL_ONE;
		currentTimeRPage = RPAGE_TIME_ONE;
		currentTuneRPage = RPAGE_TUNE_ONE;
	}else{
		currentFuelRPage = RPAGE_FUEL_TWO;
		currentTimeRPage = RPAGE_TIME_TWO;
		currentTuneRPage = RPAGE_TUNE_TWO;
	}

	RPAGE = currentTuneRPage;
}

/* Reset all state to non running */
void resetToNonRunningState(){
	/* Reset RPM to zero */
	RPM[recordADCBank] = 0;

	/* Ensure tacho reads lowest possible value */
	engineCyclePeriod = ticksPerCycleAtOneRPM;

	/* Clear all sync flags to lost state */
	//coreStatusA &= ~RPM_VALID;
	coreStatusA &= ~PRIMARY_SYNC;
	//coreStatusA &= ~SECONDARY_SYNC;

	// TODO more stuff needs resetting here, but only critical things.
}


/* Demonstrate basic PWM usage */
void adjustPWM(){
	PWMDTY0 = ATD0DR0 >> 2; // scale raw adc to a duty
	PWMDTY1 = ATD0DR1 >> 2; // scale raw adc to a duty
	PWMDTY2 = ATD0DR2 >> 2; // scale raw adc to a duty
	PWMDTY3 = ATD0DR3 >> 2; // scale raw adc to a duty
	PWMDTY4 = ATD0DR4 >> 2; // scale raw adc to a duty
	PWMDTY5 = ATD0DR5 >> 2; // scale raw adc to a duty
	PWMDTY6 = ATD0DR6 >> 2; // scale raw adc to a duty
	//PWMDTY7 = ATD0DR7 >> 2; // scale raw adc to a duty (user led instead)
}


/* Read ADCs into the correct bank one at a time linearly */
void sampleEachADC(ADCArray *Arrays){
	/* ATD0 */
	Arrays->IAT[recordADCBank] = ATD0DR0;
	Arrays->CHT[recordADCBank] = ATD0DR1;
	Arrays->TPS[recordADCBank] = ATD0DR2;
	Arrays->EGO[recordADCBank] = ATD0DR3;
	Arrays->MAP[recordADCBank] = ATD0DR4;
	Arrays->AAP[recordADCBank] = ATD0DR5;
	Arrays->BRV[recordADCBank] = ATD0DR6;
	Arrays->MAT[recordADCBank] = ATD0DR7;

	/* ATD1 */
	Arrays->EGO2[recordADCBank] = ATD1DR0;
	Arrays->IAP[recordADCBank] = ATD1DR1;
	Arrays->MAF[recordADCBank] = ATD1DR2;
	Arrays->SpareADC3[recordADCBank] = ATD1DR3;
	Arrays->SpareADC4[recordADCBank] = ATD1DR4;
	Arrays->SpareADC5[recordADCBank] = ATD1DR5;
	Arrays->SpareADC6[recordADCBank] = ATD1DR6;
	Arrays->SpareADC7[recordADCBank] = ATD1DR7;
}


/* Read ADCs into the correct bank in a loop using pointers */
void sampleLoopADC(ADCArray *Arrays){
	// get the address of the ADC array
	unsigned short addr = (unsigned short)Arrays;

	//sendUS(addr);
	unsigned char loop;
	/* (value of((address of ADCArrays struct) + (offset to start of bank(0 or half struct length)) + (offset to particular ADC (loopcounter * 4)) + (offset to correct element(0 or 2)))) =
	 * (value of((address of ARRAY block) + (loop counter * 2))) */

	for(loop=0;loop<ADCS_IN_ADC0;loop++){
		/* Do the first block */
		DVUSP(addr + /*(0) +*/ (loop * 4) + (recordADCBank * 2)) = DVUSP(ATD0_BASE + (loop * 2));

		/* Do the second block */
		DVUSP(addr + 32 + (loop * 4) + (recordADCBank * 2)) = DVUSP(ATD1_BASE + (loop * 2));
	}
}


/* Loop repeatedly for X milli seconds. */
void sleep(unsigned short ms){
	unsigned short j, k;
	for(j=0;j<ms;j++)
		for(k=0;k<5714;k++);
}


/* Loop repeatedly for X micro seconds. */
void sleepMicro(unsigned short us){ /* Very approximate... */
	unsigned short j, k;
	for(j=0;j<us;j++)
		for(k=0;k<6;k++);
}


// TODO UGLY as sin, duplication here, not sure about types etc, will fix later if possible.

// It seems to me that a LONG would do for all types, but would produce a long or variable string which is not ideal.
// It would also be horribly slow being non native. Perhaps this style of separate implementation is best?

/* Generate the ascii representation of a char */
void itoaUC(unsigned char n, char *s){
    unsigned char j;
    for(j=0;j<3;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[2 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
}


/* Generate the ascii representation of a short */
void itoaUS(unsigned short n, char *s){
    unsigned char j;
    for(j=0;j<5;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[4 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
}


/* Generate the ascii representation of a long */
void itoaUL(unsigned long n, char *s){
    unsigned char j;
    for(j=0;j<10;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[9 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
}
